module function
  implicit none
contains
  function f(x) !被積分関数
    double precision, intent(in) :: x
    double precision :: f
    f = 1.0d0/(1.0d0 + x)
    return
  end function f
end module function

program exercise20
  use function ! モジュールfunctionを呼び出す
  implicit none  

  integer :: i, k, n
  double precision :: h, x, trapezoidal, factor_tr, simpson, factor_smp, simpson38, factor_38, boole, factor_boo
 
  print *, "analytic solution: ", log(2.0d0)

  !表の見出しを出力
  write(*,*) "    n        trapeziodal      simpson       simson3/8       boole"
  
  do i = 0, 6

     n = 12*10**i ! nは3と4の倍数でないと公式が使えない

     h = 1.0 / dble(n)

     !積分値の初期化
     trapezoidal = 0.0d0
     simpson     = 0.0d0
     simpson38   = 0.0d0
     boole       = 0.0d0

     do k = 0, n

        x = h*k  !xは[0,1]をn等分した等間隔点

        ! 台形公式の重みを計算        
        factor_tr = 1.0d0
        if( k .eq. 0 .or. k .eq. n) factor_tr = 0.50d0 !両端では1/2、その他では1

        ! simpson公式の重みを計算
        if( k .eq. 0 .or. k .eq. n) then
           factor_smp = 1.0d0 !両端では1
        else if( mod(k, 2) .eq. 0) then !両端を除く偶数点
           factor_smp = 2.0d0
        else ! kが奇数のとき
           factor_smp = 4.0d0 
        end if

        ! simpson 3/8公式での重み
        if( k .eq. 0 .or. k .eq. n) then !両端
           factor_38 = 1.0d0
        else if( mod(k,3) .eq. 0) then ! 両端以外でkが3の倍数のとき
           factor_38 = 2.0d0
        else !その他の点
           factor_38 = 3.0d0
        end if

        ! booleの公式での重み
        if( k .eq. 0 .or. k .eq. n) then !両端
           factor_boo = 7.0d0
        else if( mod(k,4) .eq. 1) then ! k=1, 5, 9, ...
           factor_boo = 32.0d0
        else if( mod(k,4) .eq. 2) then ! k=2, 6, 10,...
           factor_boo = 12.0d0
        else if( mod(k,4) .eq. 3) then ! k = 3, 7, 11, ...
           factor_boo = 32.0d0
        else ! kが4の倍数のとき
           factor_boo = 14.0d0
        end if

        ! 重みをかけて和を計算
        trapezoidal = trapezoidal + factor_tr  * f(x) 
        simpson     = simpson     + factor_smp * f(x)
        simpson38   = simpson38   + factor_38  * f(x)
        boole       = boole       + factor_boo * f(x)
        
     end do

     ! kによらない因子をかける
     trapezoidal = trapezoidal * h
     simpson     = simpson     * h / 3.0d0
     simpson38   = simpson38   * h * 3.0d0/8.0d0
     boole       = boole       * h * 2.0d0/45.0d0

     ! 結果の出力
     write(*,'(i10,4es15.5)') n, abs(trapezoidal - log(2.0d0)), abs(simpson - log(2.0d0)), &
          & abs(simpson38 - log(2.0d0)), abs(boole - log(2.0d0))
     
  end do
  
end program exercise20
